@@ -48,6 +48,8 @@ mapmap CLASSTN*:paths[]   @stmt   @token @new
 weave ANY:aspect CMETHODN:method @stmt @token @new
 aspect TYPEN:type cmethod.. CLASSN:class METHODN:method STT_DECL:params {:instmt  @stmt @token @new
 
+block {:block @stmt @token @new
+
 if pexpr:bool {:truecase else.. {:falsecase  @stmt @token @new @name @cmpl
 else @token
 switch pexpr:bool {:case @token @stmt @new @name @cmpl
@@ -61,7 +63,7 @@ where @token
 
 break    any_:label ; @stmt @token @new @cmpl
 continue any_:label ;  @stmt @token @new @cmpl
-goto ANY:label ;  @stmt @token @new @cmpl
+#goto ANY:label ;  @stmt @token @new @cmpl
 
 try {:try catch.. {:catch {:finally   @stmt  @token @new  @name @cmpl
 catch EXPTN:type VARN:name {:catch   @token @stmt
@@ -70,12 +72,10 @@ throw expr:expt ;   @stmt  @token @new @name @cmpl
 err ANY:msg     @stmt @cmpl
 
 let      VARN:lvalue expr:rvalue     @stmt  @name @cmpl  @expr
+local    expr*:exprs[] ; @stmt @token @new @cmpl
 #letexpr  VARN:lvalue expr:rvalue     @stmt  @name @cmpl  @expr
 letmulti VARN*:names[]               @stmt    
 
-#next expr:value                  @stmt  @expr
-#prev expr:value                  @stmt  @expr
-
 @expr new     METHODN:new CLASSN:class expr*:args[]  @stmt @cmpl
 @expr mapcast CLASSN:class expr:source expr:option   @stmt @cmpl
 @expr call    METHODN:method expr:base expr*:args[]  @stmt @cmpl
@@ -84,12 +84,12 @@ letmulti VARN*:names[]               @stmt
 @expr mt      MT:mt expr:target expr:option @stmt  @cmpl
 @expr op      expr:first expr:second expr*:args[] @stmt  @cmpl
 
-@expr trinary expr:bool expr:tvalue expr:fvalue @stmt  @cmpl
+@expr tri     expr:bool expr:tvalue expr:fvalue @stmt  @cmpl
 @expr alt     expr*:bools[]  @stmt   @cmpl
 @expr and     expr*:bools[]  @stmt   @cmpl
 @expr or      expr*:bools[]  @stmt   @cmpl
 
-print  expr*:values[]  @stmt @token @new  @cmpl
+print  expr*:values[] ;  @stmt @token @new  @cmpl
 assert pexpr:bool ;    @token @stmt @new 
 #utest  pexpr:bool {:testcase  @stmt @test
 #man   ANY:keyword ;  @stmt @new
@@ -179,20 +179,26 @@ CONSTN                 @system @top @const @P99
 TYPEN                  @system @top @const @P99
 CMETHODN               @system @top @const @P99
 NAME                   @system @top @const @P99
-MN                     @system @P99
-FN                     @system @P99
-CID                    @system @P99
-CONST                  @system @top @const @typed @P99
-MTDMPR                 @system @top @typed @P99
-EBPIDX                 @system @typed @P99
-SFPIDX                 @system @typed @P99
-FLDIDX                 @system @typed @P99
-OBJIDX                 @system @typed @P99
-SYSCONST               @system @typed @P99
-_ ASIS                 @top    @typed
 ERR                    @system 
+MN                     @system @top @P99
+FN                     @system @top @P99
+CID                    @system @top @P99
+MPR                    @system @top @P99
 EOT                    @system @eos
 
+#Typed Token
+CONST                  @system @P99
+DEFVAL                 @system @P99
+SYSVAL                 @system @P99
+STACK                  @system @P99
+FIELD                  @system @P99
+GLOBAL                 @system @P99
+MEMBER                 @system @P99
+CLASSID                @system @P99
+_ ASIS                 @system @P99
+NOP                    @system @P99
+
+
 public  TT_METAN        @alias
 private TT_METAN        @alias
 final   TT_METAN        @alias
@@ -205,11 +211,14 @@ mod     TT_MOD          @alias
 #void    TT_TYPEN        @alias
 #any     TT_TYPEN        @alias
 
+char    Int!            @alias
+short   Int!            @alias
 int     Int!            @alias
 long    Int!            @alias
 float   Float!          @alias
 double  Float!          @alias
 boolean Boolean!        @alias
+string  String!         @alias
 def     Any             @alias
 ArrayList Array         @alias
 Integer Int             @alias
@@ -454,27 +463,6 @@ static %s knh_%s_array_%s[] = {''' % (ctype, ttype, fn))
 
 def write_knh_StmtDONE(f):
     f.write('''
-#define knh_StmtEXPR_decl knh_StmtDONE_decl
-#define knh_StmtEXPR_name knh_StmtDONE_name
-void knh_StmtEXPR_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level);
-
-static
-void knh_StmtDONE_decl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
-{
-
-}
-
-static
-void knh_StmtDONE_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
-{
-
-}
-
-static
-void knh_StmtDONE_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
-{
-
-}
 ''')
 
 #------------------------------------------------------------------------------
@@ -519,10 +507,10 @@ int knh_bytes_istoken(knh_bytes_t t, int ch)
 
 def write_tparser_c(f):
     write_chapter(f, '[token parser]')
-    CPROTO.append('Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_t line, knh_bytes_t token, BytesConv *bconv);')
+    CPROTO.append('Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t token);')
     f.write('''
 PUBLIC
-Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_t line, knh_bytes_t token, BytesConv *bconv)
+Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, InputStream *in, knh_bytes_t token)
 {
     knh_bytes_t t = token;
     knh_token_t tt = ((knh_token_t)-1);
@@ -581,10 +569,10 @@ Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line
     /* built-in */
     }
     if(tt == ((knh_token_t)-1)) {
-        return new_Token__name(ctx, flag, fileid, line, t);
+        return new_Token__NAME(ctx, flag, in, t);
     }
     else {
-        Token *tk = new_Token(ctx, flag, fileid, line, tt);
+        Token *tk = new_Token(ctx, flag, DP(in)->fileid, DP(in)->line, tt);
         if(tt == TT_METAN) {
             KNH_SETv(ctx, DP(tk)->data, new_String(ctx, t, NULL));
         }
@@ -600,9 +588,9 @@ Token *new_Token__parse(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line
 def write_stmt_add(f, token):
     write_section(f, '[%s]' % token)
     func = 'void knh_Stmt_add_%s(Ctx *ctx, Stmt *o, knh_tokens_t *tc)' % token
-    CPROTO.append('%s;' % func)
+    #CPROTO.append('%s;' % func)
     f.write('''
-%s
+static %s
 {
 \tif(SP(o)->stt == STT_ERR) return;
 \tif(tc->c < tc->e && knh_Token_is%s(tc->ts[tc->c])) {
@@ -618,8 +606,9 @@ def write_stmt_add(f, token):
 def write_new_Stmt(f, st):
     write_section(f, '[%s]' % st.name)
     if st.name.startswith("using "):
-        CPROTO.append('Stmt *new_%s(Ctx *ctx, knh_tokens_t *tc);' % st.funcbase)
+        #CPROTO.append('Stmt *new_%s(Ctx *ctx, knh_tokens_t *tc);' % st.funcbase)
         f.write('''
+static
 Stmt *new_%s(Ctx *ctx, knh_tokens_t *tc)
 {
 \tStmt *o = new_Stmt(ctx, 0, %s);''' % (st.funcbase, st.STT))
@@ -673,10 +662,10 @@ def write_sparser_c(f):
 
     write_section(f, '[stmt1]')
 
-    CPROTO.append('Stmt *new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc);')
+    #CPROTO.append('Stmt *new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc);')
     f.write('''
-Stmt *
-new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
+static
+Stmt *new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
 {
     Token *tkc = NULL;
     L_TAIL:;
@@ -791,7 +780,7 @@ def write_visitfunc(f, st, visit, rtype = 'void', args = 'Cmpl *cmpl, NameSpace
 #------------------------------------------------------------------------------
 
 def gen_stmt(bdir):
-    fname = '%s/konoha/gen/konohac_.c' % bdir
+    fname = '%s/src/gen/konohac_.c' % bdir
     f = open_c(fname, ['<konoha/konoha_dev.h>'])
     
     write_chapter(f,'[token]')
@@ -806,13 +795,13 @@ def gen_stmt(bdir):
     write_chapter(f,'[stmt]')
     write_array(f, 'char*', 'stmt', 'tochar', STT_LIST, '@tochar')
     write_array(f, 'size_t', 'stmt', 'size', STT_LIST, '@size')
-    write_sparser_c(f)
+    #write_sparser_c(f)
 
     write_knh_StmtDONE(f)
     write_array(f, 'int', 'stmt', 'isExpr', STT_LIST, '@expr')
-    write_array(f, 'f_visitdecl', 'stmt', 'decl', STT_LIST, '@decl')
-    write_array(f, 'f_visitname', 'stmt', 'name', STT_LIST, '@name')
-    write_array(f, 'f_visitcmpl', 'stmt', 'cmpl', STT_LIST, '@cmpl')
+    #write_array(f, 'f_visitdecl', 'stmt', 'decl', STT_LIST, '@decl')
+    #write_array(f, 'f_visitname', 'stmt', 'name', STT_LIST, '@name')
+    #write_array(f, 'f_visitcmpl', 'stmt', 'cmpl', STT_LIST, '@cmpl')
     
     close_c(f, fname)
     ##
