@@ -49,7 +49,6 @@ extern "C" {
 #define IS_FALSE(o)        (o == KNH_FALSE)
 #define BOOL_ISTRUE(o)         (o == KNH_TRUE)
 #define BOOL_ISFALSE(o)        (o == KNH_FALSE)
-#define new_Boolean(ctx,b)     ((b) ? KNH_TRUE : KNH_FALSE)
 #define knh_Bytes_size(o)      (o)->size
 #define knh_Bytes_value(o)     (o)->buf
 #define knh_Bytes_last(o)      ((o)->buf + (o)->size)
@@ -67,14 +66,14 @@ extern "C" {
 #define IS_SPEC(o)   (CLASS_IntUnit <= knh_Object_cid(o)  && knh_Object_cid(o) <= CLASS_Vocabulary)
 #define KNH_FLAG_TF_NN                KNH_FLAG_T0
 #define TYPE_ISNULLABLE(t)            ((t & KNH_FLAG_TF_NN)==0)
-#define TYPE_ISNOTNULL(t)             ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
-#define CLASS_TONNTYPE(c)             (c|KNH_FLAG_TF_NN)
+#define IS_NNTYPE(t)             ((t & KNH_FLAG_TF_NN)==KNH_FLAG_TF_NN)
+#define NNTYPE_cid(c)             (c|KNH_FLAG_TF_NN)
 #define TYPE_TONNTYPE(t)              (t|KNH_FLAG_TF_NN)
 #define TYPE_UNMASK(t)                (t&(~(KNH_FLAG_TF_NN|KNH_FLAG_T1)))
-#define TYPE_UNMASK_NN(t)             (t&(~KNH_FLAG_TF_NN))
+#define CLASS_type(t)             (t&(~KNH_FLAG_TF_NN))
 #define TYPEQN(t)                     TYPEN(t), TYPEQ(t)
-#define TYPE_void                     CLASS_TONNTYPE(CLASS_Nue)
-#define NNTYPE_void                   CLASS_TONNTYPE(CLASS_Nue)
+#define TYPE_void                     NNTYPE_cid(CLASS_Nue)
+#define NNTYPE_void                   NNTYPE_cid(CLASS_Nue)
 #define TYPE_any                      CLASS_Any
 #define NNTYPE_any                    CLASS_any
 #define TYPEN(type)  knh_TYPEN(ctx,type)
@@ -82,8 +81,6 @@ extern "C" {
 #define knh_ClassMap_struct_compare  NULL
 #define knh_tstruct_isNative(sid)   (sid < KONOHA_TSTRUCT_SIZE)
 #define knh_ClassStruct_struct_copy    NULL
-#define knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
-#define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
 #define knh_Connection_struct_copy    NULL
 #define knh_Connection_struct_compare  NULL
 #define KNH_STDIN          (((Context*)ctx)->in)
@@ -135,15 +132,15 @@ extern "C" {
 #define knh_Mapper_struct_copy NULL
 #define knh_Method_mn(mtd)    DP(mtd)->mn
 #define knh_Method_mf(mtd)    DP(mtd)->mf
-#define knh_Method_rtype(mtd)          knh_MethodField_rtype(knh_Method_mf(mtd))
+#define knh_Method_rtype(mtd)          knh_MethodField_rztype(knh_Method_mf(mtd))
 #define knh_Method_psize(mtd)      knh_MethodField_psize(knh_Method_mf(mtd))
-#define knh_Method_ptype(mtd,n)          knh_MethodField_ptype(knh_Method_mf(mtd),n)
+#define knh_Method_ptype(mtd,n)          knh_MethodField_pztype(knh_Method_mf(mtd),n)
 #define knh_Method_pfields(mtd,n)   knh_MethodField_pfields(knh_Method_mf(mtd),n)
 #define knh_Method_lastfield(mtd)   knh_MethodField_lastfield(knh_Method_mf(mtd))
 #define knh_NameSpace_struct_copy  NULL
 #define knh_NameSpace_struct_compare NULL
 #define CTXCLASSN(cid)     knh_Context_CLASSN(ctx,cid)
-#define CTXTYPEQN(cid)     knh_Context_CLASSN(ctx,TYPE_UNMASK_NN(cid)), TYPEQ(cid)
+#define CTXTYPEQN(cid)     knh_Context_CLASSN(ctx,CLASS_type(cid)), TYPEQ(cid)
 #define IS_NULL(o)          (((Object*)o)->h.cid == CLASS_Nue)
 #define IS_NOTNULL(o)       (((Object*)o)->h.cid != CLASS_Nue)
 #define new_Nue__b(ctx, txt)   new_Nue(ctx, new_String(ctx, txt, NULL));
@@ -161,7 +158,6 @@ extern "C" {
 #define knh_write__e(ctx, w, f)  knh_write__ffmt(ctx, w, KNH_FLOAT_FMTE, f)
 #define knh_write__fn(ctx, w, fn)   knh_write__s(ctx, w, FIELDN(fn))
 #define knh_write__O(ctx, w, o)    knh_format(ctx, w, METHODN__k, o, KNH_NULL)
-#define knh_Object_toString(ctx, b)   knh_Object_movableText(ctx, b, METHODN__s, KNH_NULL)
 #define knh_ResultSet_struct_copy NULL
 #define knh_ResultSet_struct_compare NULL
 #define knh_Runtime_struct_copy  NULL
@@ -174,39 +170,55 @@ extern "C" {
 #define knh_StringUnit_struct_compare  NULL
 #define NSN_main           0
 #define FILEN_unknown      0
-#define knh_Compiler_struct_copy   NULL
-#define knh_Compiler_struct_compare  NULL
+#define knh_Asm_struct_copy   NULL
+#define knh_Asm_struct_compare  NULL
+#define KNH_ASM_JMP(ctx, abr, l)  KNH_ASM_JMP_(ctx, abr, l);
 #define knh_KLRCode_struct_compare  NULL
 #define KNH_PERRATA_(ctx, f, l, msgo, msgn)    knh_perrata(ctx, f, l, msgo, msgn)
 #define KNH_PERRATA(ctx, tk, msg)  knh_perrata(ctx, (tk)->fileid, (tk)->line, knh_Token_tochar(tk), msg)
 #define knh_Stmt_struct_copy  NULL
 #define knh_Stmt_struct_compare  NULL
-#define knh_Stmt_terms_cmpl(ctx, stmt, n, cpr, ns, reqt, level)      knh_Stmt_terms_cmpl_(ctx, stmt, n, cpr, ns, reqt, level, 0)
-#define knh_Stmt_terms_cmplpush(ctx, stmt, n, cpr, ns, reqt, level)  knh_Stmt_terms_cmpl_(ctx, stmt, n, cpr, ns, reqt, level, 1)
 #define knh_Token_struct_copy  NULL
 #define knh_Token_struct_compare  NULL
+#define new_TermCONST(ctx, fln, d)   TM(new_TokenCONST(ctx, fln, d))
+#define TERMs_getcid(stmt, n)    CLASS_type(TERMs_gettype(stmt, n))
+#define TERMs_getbcid(stmt, n)   knh_tClass[TERMs_getcid(stmt,n)].bcid
+#define KNH_OUTERPARAMS -1
+#define KNH_INNERPARAMS -2
 #define KNH_FLAG_CF2OF(f)        (f)
 #define knh_tClass_isPMZ(cid)    (knh_tClass[cid].p1 != CLASS_Nue)
-#define DEBUG_ASSERT_cid(cid)    KNH_ASSERT(IS_cid(cid))
+#define KNH_ASSERT_cid(cid)    KNH_ASSERT(IS_cid(cid))
 #define CLASSN(cid)   knh_String_tochar(knh_tClass[cid].lname)
 #define CLASSNo(o)    knh_String_tochar(knh_tClass[(o)->h.cid].lname)
+#define KNH_DEF(ctx, cid)  knh_tClass_defaultValueNN(ctx, cid)
+#define KNH_SYS(ctx, n)    knh_tConst_systemValue(ctx, n)
+#define KNH_SYS_CTX    0
+#define KNH_SYS_STDIN  1
+#define KNH_SYS_STDOUT 2
+#define KNH_SYS_STDERR 3
+#define KNH_SYS_OS     4
+#define KNH_SYS_SCRIPT 5
 #define EXPT_unknown  ((knh_expt_t)-1)
 #define EXPT_newid    ((knh_expt_t)0)
 #define DEBUG_ASSERT_eid(cid)    KNH_ASSERT(IS_eid(cid))
 #define EXPTN(eid)   knh_String_tochar(knh_tExpt_name(eid))
 #define KNH_TMAPMAP  KNH_TMAPPER
+#define knh_tMapper_find(ctx, scid, tcid) knh_tMapper_find_(ctx, scid, tcid, 1)
+#define knh_Class_getMapper(ctx, scid, tcid)  knh_tMapper_find_(ctx, scid, tcid, 0)
+#define knh_tMethodField_size               knh_Array_size(DP(knh_rootSystem)->tMethodFields)
+#define knh_Class_getMethod(ctx, c, mn)    knh_Class_getMethod__(ctx, c, mn, 0)
+#define knh_Class_findMethod(ctx, c, mn)   knh_Class_getMethod__(ctx, c, mn, 1)
 #define FIELDN_NONAME    ((knh_fieldn_t)-1)
 #define FIELDN_NEWID     ((knh_fieldn_t)-2)
 #define FIELDN_return    FIELDN_
 #define METHODN_NONAME   ((knh_methodn_t)-1)
 #define METHODN_NEWID    ((knh_methodn_t)-2)
-#define KNH_FLAG_FN_SUPER   KNH_FLAG_T0
-#define KNH_FLAG_FN_U1      KNH_FLAG_T1
-#define KNH_FLAG_FN_U2      KNH_FLAG_T2
+#define KNH_FLAG_FN_U1         KNH_FLAG_T0
+#define KNH_FLAG_FN_U2         KNH_FLAG_T1
+#define KNH_FLAG_FN_SUPER      (KNH_FLAG_T0|KNH_FLAG_T1)
 #define FIELDN_IS_SUPER(fnq)       ((fnq & KNH_FLAG_FN_SUPER) == KNH_FLAG_FN_SUPER)
 #define FIELDN_IS_U1(fnq)          ((fnq & KNH_FLAG_FN_U1) == KNH_FLAG_FN_U1)
 #define FIELDN_IS_U2(fnq)          ((fnq & KNH_FLAG_FN_U2) == KNH_FLAG_FN_U2)
-#define FIELDN_IS_PROTECTED(fnq)   (FIELDN_IS_U1(fnq)||FIELDN_IS_U2(fnq))
 #define FIELDN_UNMASK(fnq)         (fnq & (~(KNH_FLAG_FN_SUPER|KNH_FLAG_FN_U1|KNH_FLAG_FN_U2)))
 #define KNH_FLAG_MN_SUPER        KNH_FLAG_T0
 #define KNH_FLAG_MN_GETTER       KNH_FLAG_T1
@@ -229,9 +241,6 @@ extern "C" {
 #define knh_systemStdIn       DP(knh_rootSystem)->in
 #define knh_systemStdOut      DP(knh_rootSystem)->out
 #define knh_systemStdErr      DP(knh_rootSystem)->err
-#define knh_tMethodField_add(ctx, mf)       knh_Array_add(ctx, DP(knh_rootSystem)->tMethodFields, UP(mf))
-#define knh_tMethodField(n)                 (MethodField*)knh_Array_n(DP(knh_rootSystem)->tMethodFields, n)
-#define knh_tMethodField_size               knh_Array_size(DP(knh_rootSystem)->tMethodFields)
 #define new_Int(ctx, n)      new_Int__fast(ctx, (n))
 #define new_Float(ctx, n)      new_Float__fast(ctx, (n))
 #define new_String__T(ctx, text)    new_StringX__T(ctx, CLASS_String, text)
@@ -243,31 +252,11 @@ extern "C" {
 #define STRUCT_UNMASK(sid)         (sid & (~KNH_FLAG_SF_FIELD))
 #define DEBUG_ASSERT_sid(sid)   KNH_ASSERT(IS_sid(sid))
 #define STRUCTN(sid)   knh_tstruct_name(sid)
-#define KNH_LOPEN(ctx, n) \
-	knh_sfp_t *lbp_ = ((Context*)ctx)->ebp + 1; \
-	knh_stackf_locals(ctx, n) \
-
-
-#define KNH_LOCALn(n)                 (lbp_[n].o)
-#define KNH_LSETv(ctx, n, v)          KNH_SETv(ctx, lbp_[n].o, v)
-#define KNH_LCLOSE(ctx)               ((Context*)ctx)->ebp = lbp_ - 1
-#define KNH_LPUSH(ctx,v) \
-	((Context*)ctx)->ebp++; \
-	KNH_SETv(ctx, ((Context*)ctx)->ebp[0].o, v); \
-
-
-#define KNH_PUSHv(ctx,v) \
-	((Context*)ctx)->ebp++; \
-	KNH_SETv(ctx, ((Context*)ctx)->ebp[0].o, v); \
-*/
-
 #define KNH_FLAG_PF_STDERR      KNH_FLAG_T0
 #define KNH_FLAG_PF_EOL         KNH_FLAG_T1
 #define KNH_FLAG_PF_TIME        KNH_FLAG_T2
 #define KNH_FLAG_PF_FUNC        KNH_FLAG_T3
 #define KNH_FLAG_PF_NAME        KNH_FLAG_T4
-#define KNH_FLAG_PF_NOCOMMA     KNH_FLAG_T5
-#define KNH_FLAG_PF_EMPTY       KNH_FLAG_T6
 #define KNH_FLAG_PF_BOL         KNH_FLAG_T7
 #define KNH_THROW(ctx, e) knh_throw(ctx, UP(e), KNH_SAFEFILE(__FILE__), __LINE__)
 #define KNH_THROWs(ctx, s) \
@@ -284,6 +273,7 @@ extern "C" {
 
 /* ======================================================================== */
 /* [PROTOTYPE] */
+KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv);
 KNHAPI(void) konoha_addAffineMapper(Ctx *ctx, knh_class_t scid, char *text, knh_float_t scale, knh_float_t shift);
 KNHAPI(knh_Array_t*) new_Array(Ctx *ctx, knh_class_t p1, size_t capacity);
 KNHAPI(void) knh_Array_add(Ctx *ctx, Array *o, Any *value);
@@ -328,7 +318,7 @@ KNHAPI(void) knh_println(Ctx *ctx, OutputStream *w, knh_bytes_t s);
 KNHAPI(void) knh_write_EOL(Ctx *ctx, OutputStream *w);
 KNHAPI(void) knh_write_TAB(Ctx *ctx, OutputStream *w);
 KNHAPI(void) knh_write_BOL(Ctx *ctx, OutputStream *w);
-KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Any *x, Any *m);
+KNHAPI(void) knh_format(Ctx *ctx, OutputStream *w, knh_methodn_t mn, Object *o, Any *m);
 KNHAPI(void) knh_printf(Ctx *ctx, OutputStream *w, char *fmt, ...);
 KNHAPI(void) konoha_says(Ctx *ctx, int type, char *fmt, ...);
 KNHAPI(void) knh_ResultSet_initColumn(Ctx *ctx, ResultSet *o, size_t column_size);
@@ -343,7 +333,6 @@ KNHAPI(void) konoha_addAliasURN(Ctx *ctx, char *alias, char *urn);
 KNHAPI(String*) new_String(Ctx *ctx, knh_bytes_t t, String *orign);
 KNHAPI(knh_bytes_t) knh_String_tobytes(String *o);
 KNHAPI(knh_bytes_t) knh_StringNULL_tobytes(String *o, knh_bytes_t def);
-KNHAPI(String*) new_String_bconv(Ctx *ctx, String *s, f_bconv bconv);
 KNHAPI(void) konoha_addParserDriver(Ctx *ctx, char *alias, knh_parser_drvapi_t *d);
 KNHAPI(ClassSpec*) new_Vocabulary(Ctx *ctx, char *lang, char *defval, ...);
 KNHAPI(void) konoha_addVocabularyMapper(Ctx *ctx, knh_class_t scid, char *text);
@@ -356,7 +345,8 @@ KNHAPI(int) konoha_issweep(knh_ftraverse ftr);
 KNHAPI_(Int*) new_Int__fast(Ctx *ctx, knh_integer_t value);
 KNHAPI(Int*) new_IntX__fast(Ctx *ctx, knh_class_t cid, knh_integer_t value);
 KNHAPI_(Float*) new_Float__fast(Ctx *ctx, knh_float_t value);
-KNHAPI(knh_boolean_t) knh_System_hasLibrary(Ctx *ctx, char *libname, char *funcname);
+KNHAPI_(Object*) knh_boxing(Ctx *ctx, knh_sfp_t *sfp, knh_class_t cid);
+KNHAPI(int) knh_System_hasLibrary(Ctx *ctx, char *libname, char *funcname);
 KNHAPI(void) konoha_addClassSpecFunc(Ctx *ctx, char *urn, f_makespec func);
 KNHAPI(int) konoha_setopt(int argc, char **argv);
 KNHAPI(void) konoha_context_initOption(Ctx *ctx);
@@ -384,6 +374,10 @@ KNHAPI(void) konoha_addStringConst(Ctx *ctx, char *name, char *text);
 KNHAPI(void) konoha_addDriver(Ctx *ctx, char *name, int type, void *drv);
 KNHAPI(char*) KNH_SAFEFILE(char *file);
 KNHAPI(void) konoha_shell(Ctx *ctx);
+KNHAPI_(void) knh_sfp_boxing(Ctx *ctx, knh_sfp_t *sfp);
+KNHAPI_(void) knh_sfp_unboxing(Ctx *ctx, knh_sfp_t *sfp);
+KNHAPI(void) knh_sfp_sformat(Ctx *ctx, knh_sfp_t *sfp, Method *mtd, OutputStream *w, Any *m);
+KNHAPI(void) knh_sfp_format(Ctx *ctx, knh_sfp_t *sfp, knh_methodn_t mn, OutputStream *w, Any *m);
 KNHAPI(void) knh_throw_TODO(Ctx *ctx, char *file, int line, char *func);
 KNHAPI(char*) konoha_doing(Ctx *ctx, char *msg);
 KNHAPI(void) knh_throwException(Ctx *ctx, Exception *e, char *file, int line);
