@@ -38,7 +38,8 @@
 #define KONOHA_VERSION   PACKAGE_VERSION
 #else
 #define KONOHA_NAME     "konoha"
-#define KONOHA_VERSION  "0.2.5-rc2"
+#define KONOHA_MAJOR_VERSION "0.3"
+#define KONOHA_VERSION  "0.3"
 #endif
 
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
@@ -51,7 +52,7 @@
 /* [OPTIONS] */
 
 #define KNH_USING_RCGC    1                /* Reference Counting */
-#define KNH_USING_DRCGC   1
+//#define KNH_USING_DRCGC   1
 
 /* ======================================================================== */
 
