@@ -201,21 +201,11 @@
 /* ======================================================================== */
 
 #ifdef KNH_USING_POSIX
-	#include<dlfcn.h>              /* @using dl -ldl */
-	#define knh_dlopen(p,m)         dlopen(p,m)
-	#define knh_dlsym(hdr,name)     dlsym(hdr,name)
-	#define knh_dlerror()           dlerror()
-	#define knh_dlclose(hdr)        dlclose(hdr)
+#include<dlfcn.h>
 #define KNH_RTLD_LAZY               RTLD_LAZY
 #else
-	#define knh_dlopen(p,m)         ALT_dlopen(p,m)
-	#define knh_dlsym(hdr,name)     ALT_dlsym(hdr,name)
-	#define knh_dlerror()           ALT_dlerror()
-	#define knh_dlclose(hdr)        ALT_dlclose(hdr)
-#ifndef KNH_RTLD_LAZY
 #define KNH_RTLD_LAZY               0
 #endif
-#endif
 
 /* ------------------------------------------------------------------------ */
 
