@@ -55,6 +55,8 @@ extern "C" {
 #define PRIVATE            static
 
 #define METHOD             void  KNH_CC_FASTCALL
+#define MAPPER             METHOD
+#define ITRNEXT            METHOD
 
 #ifdef KNH_USING_THREAD
 #define KNH_MT_VOLATILE           volatile
@@ -301,9 +303,8 @@ typedef knh_ushort_t          knh_methodn_t;
 #define KNH_OBJECT_MAGIC         578
 #define KNH_FASTMALLOC_MAGIC     911
 
-#ifdef KONOHA_WITH_RCGC
+#ifdef KNH_USING_RCGC
 #define KNH_HOBJECT_REFC      1
-#define KNH_HOBJECT_REFC_INIT 0
 #endif
 
 typedef struct knh_hObject_t {
@@ -314,10 +315,6 @@ typedef struct knh_hObject_t {
 #ifdef KNH_HOBJECT_REFC
 	KNH_MT_VOLATILE knh_uint_t refc;
 #endif
-#ifdef KNH_HOBJECT_ZCTFLAG
-	knh_Object_t *buf;
-	volatile knh_uint_t dirty;       /* Don't remove this until you draw something. */
-#endif
 } knh_hObject_t ;
 
 typedef struct knh_Object_t {
@@ -353,38 +350,127 @@ typedef struct knh_Object_t {
 /* Context */
 /* ------------------------------------------------------------------------ */
 
-#define ARG_size(a)        ((size_t)(a.i)->value)
-#define ARG_int(a)         ((knh_int_t)(a.i)->value)
-#define ARG_uint(a)        ((knh_uint_t)(a.i)->value)
-#define ARG_integer(a)     ((a.i)->value)
-#define ARG_uinteger(a)    ((knh_uinteger_t)(a.i)->value)
+typedef unsigned char           knh_code_t;          /* knh_vmc_t */
 
-#define ARG_float(a)       ((knh_float_t)(a.f)->value)
-#define ARG_double(a)      ((double)(a.f)->value)
-#define ARG_char(a)        (knh_String_tochar(a.s))
-#define ARG_bytes(a)       (knh_Bytes_tobytes(a.s))
-#define ARG_cid(a)         ((a.c)->cid)
+#define p_size(a)      ((size_t)(a).ivalue)
+#define p_int(a)         ((knh_int_t)(a).ivalue)
+#define p_uint(a)        ((knh_uint_t)(a).ivalue)
+#define p_integer(a)     ((a).ivalue)
+#define p_uinteger(a)    ((knh_uinteger_t)(a).ivalue)
+
+#define p_float(a)       ((a).fvalue)
+#define p_double(a)      ((double)(a).fvalue)
+#define p_bool(a)        ((a).bvalue)
+#define p_char(a)        (knh_String_tochar(a.s))
+#define p_bytes(a)       (knh_Bytes_tobytes(a.s))
+#define p_cid(a)         ((a.c)->cid)
 #define ARG_Object_cid(a)  knh_Object_cid(a.o)
-#define ARG_cptr(a)        ((a.n)->cptr)
+#define p_cptr(a)        ((a.n)->cptr)
 
 /* ------------------------------------------------------------------------ */
 
 typedef struct knh_sfp_t {
 	union {
+		void   *ref;
 		Object *o;
 		struct knh_Int_t    *i;
 		struct knh_Float_t  *f;
 		struct knh_Class_t  *c;
 		struct knh_String_t *s;
-		struct knh_Bytes_t  *b;
+		struct knh_Bytes_t  *ba;
+		struct knh_Iterator_t *it;
 		struct knh_COBJ_t   *n;
+		struct knh_OutputStream_t *w;
 		struct knh_Method_t *mtd;
+		struct knh_Mapper_t *mpr;
 		struct knh_Exception_t *e;
 		struct knh_ExceptionHandler_t *hdr;
 	};
-	knh_uint_t op;
+	union {
+		knh_boolean_t bvalue;
+		knh_integer_t ivalue;
+		knh_float_t   fvalue;
+		knh_code_t    *pc;
+		knh_uint_t     op;
+		knh_uint64_t  data;
+	};
 } knh_sfp_t;
 
+
+#define KNH_RETURN(ctx, sfp, v) {\
+		Int *n_ = (Int*)v;\
+		KNH_NGCMOV(ctx, sfp[-1].o, n_);\
+		sfp[-1].data = (n_)->n.data;\
+		return; \
+	}\
+
+#define KNH_RETURN_void(ctx, sfp)      {\
+		KNH_NGCMOV(ctx, sfp[-1].o, KNH_VOID); \
+		return; \
+	}\
+
+#define KNH_RETURN_Boolean(ctx, sfp, c) {\
+		sfp[-1].bvalue = c; \
+		return; \
+	}\
+
+#define KNH_RETURN_Int(ctx, sfp, n)      {\
+		sfp[-1].ivalue = (knh_integer_t)n;\
+		return; \
+	}\
+
+#define KNH_RETURN_Float(ctx, sfp, n)      {\
+		sfp[-1].fvalue = (knh_float_t)n;\
+		return; \
+	}\
+
+#define KNH_ITRNEXT(ctx, sfp, n, v) {\
+		KNH_MOV(ctx, sfp[n].o, v);\
+		return; \
+	}\
+
+#define KNH_ITREND(ctx, sfp, n) {\
+		KNH_MOV(ctx, sfp[n].o, KNH_VOID);\
+		return; \
+	}\
+
+#define HAS_ITRNEXT(v)   IS_NOTNULL(v)
+
+#define KNH_ITRNEXT_Int(ctx, sfp, n, value) {\
+		sfp[n].ivalue = value;\
+		KNH_MOV(ctx, sfp[n].o, KNH_INT0);\
+		return; \
+	}\
+
+#define KNH_ITREND_Float(ctx, sfp, n, value) {\
+		sfp[n].fvalue = value;\
+		KNH_MOV(ctx, sfp[n].o, KNH_FLOAT0);\
+		return; \
+	}\
+
+#define KNH_MAPPED(ctx, sfp, v) {\
+		KNH_MOV(ctx, sfp[0].o, v);\
+		return; \
+	}\
+
+#define KNH_MAPPED_Boolean(ctx, sfp, value) {\
+		sfp[0].bvalue = value;\
+		KNH_MOV(ctx, sfp[0].o, KNH_TRUE);\
+		return; \
+	}\
+
+#define KNH_MAPPED_Int(ctx, sfp, value) {\
+		sfp[0].ivalue = value;\
+		KNH_MOV(ctx, sfp[0].o, KNH_INT0);\
+		return; \
+	}\
+
+#define KNH_MAPPED_Float(ctx, sfp, value) {\
+		sfp[0].fvalue = value;\
+		KNH_MOV(ctx, sfp[0].o, KNH_FLOAT0);\
+		return;\
+	}\
+
 /* ------------------------------------------------------------------------ */
 
 typedef struct knh_Context_t {
@@ -411,7 +497,7 @@ typedef struct knh_Context_t {
 	struct knh_HashMap_t        *tmethodHashMap;
 	struct knh_NameSpace_t      *ns;
 	struct knh_DictMap_t        *tsymbolDictMap;
-	struct knh_Compiler_t       *cmpr;
+	struct knh_Asm_t            *cmpr;
 } knh_Context_t ;
 
 typedef const knh_Context_t    Ctx;
@@ -486,38 +572,6 @@ typedef knh_uint_t                knh_stackop_t;
 /* KLRCode */
 /* ------------------------------------------------------------------------ */
 
-typedef unsigned char           knhvmc_t;          /* knh_vmc_t */
-
-#ifdef KNH_USING_WORDKLRC    /*KNH_FASTMODE*/
-typedef knh_uint_t            knhvm_opcode_t;
-typedef knh_int_t             knhvm_i1_t;
-typedef knh_uint_t            knhvm_u1_t;
-typedef knh_int_t             knhvm_i2_t;
-typedef knh_uint_t            knhvm_u2_t;
-typedef knh_int_t             knhvm_ebp_t;
-typedef knh_int_t             knhvm_sfp_t;
-typedef knh_int_t             knhvm_ofp_t;
-typedef knh_int_t             knhvm_addr_t;
-typedef knh_uint_t            knhvm_class_t;
-typedef knh_uint_t            knhvm_methodn_t;
-typedef knh_uint_t            knhvm_expt_t;
-typedef knh_uint_t            knhvm_line_t;
-#else/*KNH_USING_WORDKLRC*/
-typedef unsigned char           knhvm_opcode_t;
-typedef char                    knhvm_i1_t;
-typedef unsigned char           knhvm_u1_t;
-typedef knh_short_t             knhvm_i2_t;
-typedef knh_ushort_t            knhvm_u2_t;
-typedef knhvm_u1_t              knhvm_ebp_t;
-typedef knhvm_u1_t              knhvm_sfp_t;
-typedef knhvm_u1_t              knhvm_ofp_t;
-typedef knh_short_t             knhvm_addr_t;
-typedef knh_class_t             knhvm_class_t;
-typedef knh_methodn_t           knhvm_methodn_t;
-typedef knh_expt_t              knhvm_expt_t;
-typedef knh_int_t               knhvm_line_t;
-#endif/*KNH_USING_WORDKLRC*/
-
 /* ------------------------------------------------------------------------ */
 
 typedef struct {
@@ -536,11 +590,13 @@ typedef void   (KNH_CC_FASTCALL *knh_fmethod)(Ctx *, knh_sfp_t *);
 typedef METHOD (*knh_fmethod)(Ctx *, knh_sfp_t *);
 #endif
 
-typedef void       MapData;
-struct knh_Mapper_t;
-typedef Object*    (*knh_fmapper)(Ctx *, Object *, struct knh_Mapper_t *);
+#ifdef _MSC_VER
+typedef void   (KNH_CC_FASTCALL *knh_fmapper)(Ctx *, knh_sfp_t *);
+#else
+typedef MAPPER (*knh_fmapper)(Ctx *, knh_sfp_t *);
+#endif
 
-typedef knh_ushort_t             knh_line_t;
+typedef knh_ushort_t             knh_sline_t;
 
 /* ======================================================================== */
 /* driver */
